





     'TARGET GROUP LIST EXTRACTOR/SEQUENCER     16SEP88     [File: EXTRACT.BAS]

     'Author : R J Morton
     'This module extracts a Target List of prospects whose Market Profiles
     'fall within the Target Profile whose number is TargNum. It places the
     'Target List in a file whose name and path spec is the same as that of
     'the main Target Profiles file, but with the TargNum appended as a two-
     'digit number (with leading zero where necessary). The target list con-
     'tains the main record [or Ref] number of each prospect on the list,
     'plus the elapsed times from/to his last and next events. This module
     'receives control only from DefTarg.bas to which it retruns it after
     'completing the target List extraction or until aborted.

     'INCLUDE FILES & LOCAL DIMENSIONING

     DefInt b-z                      'All variables are intergers by default
     '$include: 'ComVars.INC'     'Main common variables
     '$include: 'ComVarT.INC'     'Local group of common variables holding
     '                                the Target Profile specification.
     RecNum = 0
     dim sicn$(18)                   'Selected SIC numbers
     dim Code$(18)                   'Valid codes
     dim FileReqd(9)                 'flags showing which files are required
     dim FileOpen(9)                 'flags showing which files are open
     dim Rec$(12)                    'dummy


     'ARRAY OF FILE DETAILS FOR ALL FILES USED BY THIS MODULE

     dim FileName$(9):dim RecLen(9):dim LockFile(9)         'FILES

     FileName$(1)=NameFile$:RecLen(1)= 16:LockFile(1)=3     'Namecode
     FileName$(2)=PostFile$:RecLen(2)= 16:LockFile(2)=3     'Postcode
     FileName$(3)=KeysFile$:RecLen(3)=  8:LockFile(3)=2     'Keys
     FileName$(4)=AttrFile$:RecLen(4)= 16:LockFile(4)=2     'Attributes
     FileName$(5)=DateFile$:RecLen(5)= 16:LockFile(5)=2     'Dates
     FileName$(6)=DistFile$:RecLen(6)=  4:LockFile(6)=2     'Districts
     FileName$(7)=AreaFile$:RecLen(7)=202:LockFile(7)=2     'Areas
     FileName$(8)=CodeFile$:RecLen(8)= 16:LockFile(8)=2     'Codes
     FileName$(9)=TargFile$_
                 +TargNum$ :RecLen(9)=  8:LockFile(9)=0     'Target

     'LockFile=0 is Read-Write access, other users denied access
     'LockFile=2 is Read-only access with other users allowed to READ & WRITE
     'LockFile=3 is Read-only access with other users not permitted to WRITE
     '





     'INITIALIZATION

     DosRow= 3                          'Row on which error messages displayed
     DosCol= 1                          'Column at which error messages start
     DosLen=80                          'Length of error message display field
     CR$=chr$(13):Esc$=chr$(27)
     gosub ExpiryTest                   'Package security 
     ExtractorState=1                   'Kick-start the Extractor FMM
     if left$(TargName$,1)="-" then     'If Target Name begins with a -
        Exceptions=1                    'Set to compile the exception list
        end if


     'FMM POLLING LOOP

     'All FMMs are first-level subroutines. Error handling for disk I/O is
     'provided only for first-level subroutines. No disk I/O commands must
     'therefore appear in any higher-level subroutines.

     While QuitFlag=0
     gosub Keyboard                     'Poll the keyboard & system clock

     if KbChar$=Esc$ then QuitFlag=1    'Keyboard abort

     if KbChar$=CR$ then                'If re-start request from keyboard,
        ExtractorState=1                'Re-start the Extractor FMM
        ps$="                     "     'Clear the `re-start' message.
        row=21:column=11:gosub 50       '
        end if                          '

     gosub Extractor                    'Open all files needed for extraction
     gosub ShowMessages                 'Display any DOS error messages
     wend                               'Not quit
     ExtractorState=0
     close                              'Close all files used in the extraction
     DosMsg$="":ShowMessages=1:gosub ShowMessages
     chain"DefTarg"                     'Return to the Target Definition module


     'EXTERNAL SUBROUTINES AND FINITE STATE MACHINES

50   Call ScrSub(ps$,row,column,colour):return

     '$include: 'OpenFile.FMM'    'File Opener & error handler
     '$include: 'Elapsed.INC'     'Elapsed time calculator
     '$include: 'Keyboard.FMM'    'Character input routine
     '$include: 'Expiry.INC'      'Character input routine
     '$include: 'Decrypt.INC'     'Character input routine
     '






     'THE EXTRACTOR FINITE STATE MACHINE

     'This routine uses the FileReqd( ) array to determine which of the 9 files must
     'be opened in order for the Extractor to perform the particular extraction
     'specified in the Target Profile specification. If FileReqd(n)=1 then File#n
     'has to be opened, otherwise it can remain closed. When FileReqd(n)=1, the
     'general `OpenFile' FMM is called to open the file. `OpenFile.FMM' then
     'tries to open the required file. It attempts retries if the `open' fails
     'for reasons that could be attributable to temporary denial by a network.
     'It then proceeds with the extraction process.

     'The Extractor's machine-states are presented in order of frequency of use
     'to minimise the number of times the machine state test has to be made.


     'STATE 4: MAIN SEQUENCE OF THE EXTRACTION PROCESS

Extractor:
     IF ExtractorState=0 THEN return    'STATE 0: FINITE STATE MACHINE IDLE
     IF ExtractorState=4 THEN           'If machine is in Main Sequence
        if SeqTyp=1 goto NameSeq        'Sequence = namecode
        if SeqTyp=2 goto RefSeq         'Sequence = record number
        if SeqTyp=6 goto PostSeq        'Sequence = postcode
        goto AlphaSeq                   'Sequence = alpha within geographic
     '





     'STATE 3: WAIT FOR FILE TO OPEN, THEN SET UP THE FILE'S RECORD FIELDS

 ElseIF ExtractorState=3 THEN           'STATE 3: WAIT FOR THE FILE TO OPEN
        gosub OpenFileFMM               'Call the file opener FMM
        if OpenFileState<5 then return  'File opener not yet terminated

        if OpenFileState=5 then         'If file has been opened successfully
           OpenFileState=0              'De-activate the file opener

           if FileNum=1 then field#1, 8 as n1$,2 as n2$    _  'Set the fields
      else if FileNum=2 then field#2, 8 as p1$,2 as p2$    _  'for the file
      else if FileNum=3 then field#3, 4 as k1$,4 as k2$    _  'currently
      else if FileNum=4 then field#4,16 as aa1$            _  'being opened.
      else if FileNum=5 then field#5, 7 as d1$,7 as d2$    _  '
      else if FileNum=6 then field#6, 4 as z1$             _  '
      else if FileNum=7 then field#7, 2 as a1$,200 as a2$  _  '
      else if FileNum=8 then field#8,16 as c1$             _
      else if FileNum=9 then field#9, 2 as s1$,2 as s2$,2 as s3$

           FileOpen(FileNum)=1          'Set this file as now open
           gosub FileLamps              'Display its lamp bright green
           ExtractorState=2             'Advance to the next machine state

           else                         'If file would not open, then
           ExtractorState=0             'De-activate the main sequence
           end if
     '





     'STATE 2: THE LOOP-CONTROL STATE

 ElseIF ExtractorState=2 THEN           'STATE 2: SET UP DETAILS OF NEXT FILE

        if FileNum<9 then               'If there are still more files to open
           FileNum=FileNum+1            'Advance to the next file

           if FileReqd(FileNum)=1 then      'If this file needs to be opened,
              FileName$=            _
              FileName$(FileNum)        'Set up the file's path\name
              RecLen=RecLen(FileNum)    'Set up the file's record length
              LockFile=LockFile(FileNum)'Set for Read-only access, shared.
              OpenFileState=1           'Activate the file opener
              ExtractorState=3          'Advance to the next machine state
              end if

           else                         'else [if last file been dealt with]

           FileNum=1
           On Error goto ErrorOnGet     'Set error trap for rest of program
           FileNum=1:get#1,1
           i.max=cvi(n1$)      'Number of entries in namecode/postcode index
           r.max=cvi(n2$)      'Record number of highest occupied main record
           gosub ShowMax       'Show the above two items

           if SeqTyp=2 or SeqTyp=6 then 'For Ref N or Postcode sequencing,
              close#1                   'Namecode file only needed to load
              FileReqd(1)=0             '`i.max' Clear file #1's lamp
              end if

           ExtractorState=4             'Transmute to the Main Sequence state
           end if
     '





     'STATE 1: THE INITIALIZATION STATE

 ElseIF ExtractorState=1 THEN           'STATE 1: Initialization state

        gosub FlagSub                   'Convert Target Status word to flags
        gosub SicStrip                  'Strip & pack SIC Ns
        gosub CodeStrip                 'Strip & pack Selector Codes

        if SeqTyp<1 then SeqTyp=1       'Default to straight alphabetic order

        TargArea$=left$ (PostZone$,2)   'Limited to post area
        TargDist$=right$(PostZone$,2)   'Limited to post district

        accepts=0                       'N of prospects who fit target profile
        rejects=0                       'N of prospects who don't

        Ni=0                            'Namecode Index counter

        if SeqTyp>2 and SeqTyp<6 then   'Initialize Postcode Index
           Pi=1                         'start from first postcode
      else Pi=0                         'start by getting post index size
           end if

        GpSize=0                        'Size of current Postcode GeoGroup
        GpBase=0                        'Postcode Group Base
        GpPost$=""                      'Group Post Area Code

        gosub ShowExtrBox               'Display Extractor's display box
        gosub FileIndicator             'Show active files indicator panel
        gosub OpeningCriteria           'Find which files must be opened
        FilesOpen=1                     'Hereafter, indicate files which are.
        FileNum=0                       'Set ready to deal with first file
        ExtractorState=2                'Advance to the loop control state
     '






     'STATE 5: STORE THE TARGET LIST PARAMETERS IN RECORD#1 OF TARGET LIST FILE

 ElseIF ExtractorState=5 THEN        'If machine is in termination sequence

        TargListSerNum=        _ 'Increment the central target list Serial N
        TargListSerNum+1
        if TargListSerNum>9999 _ 'If it reaches its upper (roll-round) limit,
           then TargListSerNum=1 'then re-set it to 1.

        TargSize=accepts         'N of prospects captured on this target list
        FileSize=i.max           'Total number of prospects on file
        ExtrFlag=TargListSerNum  'A non-zero serial N means extracted OK
        PrntFlag=0               'Clear `Target list not printed' flag
        Prog$="EXTRACT"          'Plant module name only if extract successful

        rset S1$=mki$(TargSize)  '
        rset S2$=mki$(FileSize)  '
        rset S3$=mki$(ExtrFlag)  '
        rset S4$=mki$(PrntFlag)  '

        FileNum=9:put#9,1        'Store target list parameters in first record
                                 'of the appropriate target list file.
        on error goto 0          'Cancel the local error trap

        gosub Fin                'Display the extraction completion message

        QuitFlag=1               'Set the flag to quit the Extractor overlay
        ExtractorState=0         'De-activate the Main Sequencer

        END IF

     RETURN                      'Return to the FMM polling loop
     '





     'ERROR HANDLER [AT SUBROUTINE LEVEL 1]

     'This error-handling routine is entered when an error occurs in a first-
     'level subroutine (ie FMM-level).

ErrorOnGet:
     x$ = er$(err) + " at " + str$(erl) + " in "   'Note the Dos Error Message
     resume ErrorOnGet1

ErrorOnGet1:
     DosMsg$=x$+FileName$(FileNum)      'Set up the DOS error message for
     DosClr=12:ShowMessages=1           'display.
     ps$="PRESS C/R TO RESTART"         'Display the invitation to re-start
     row=21:column=11:colour=2:gosub 50  'message.
     ps$="C/R"                          'Display the invitation to re-start
     column=17:colour=10:gosub 50       'message.

RecNumError:
     On error goto 0                    'Cancel the local error trap
     ExtractorState=0                   'De-activate the Extractor
     close                              'Close all files before re-start

     for k=1 to 9
     FileReqd(k)=0                      'Clear the files required array
     FileOpen(k)=0                      'Clear the files open array
     next

     gosub FileIndicator
     gosub ErrFin                       'Display failure message
     beep:KbChar=0
     return                             'return to FMM polling loop.
     '





     'THE `NAMECODE' ALPHABETIC SEQUENCER [SeqTyp=1]

     'This sequencing routine takes each prospect main record reference number
     'in turn in alphabetic order of namecode from the Namecode Index File and
     'passes it to the Selector routine which decides whether or not to place 
     'the prospect on the Target List.  

     'Note: Namecode index entry record numbers are biased one-up because
     'Record#1 is reserved for i.max and r.max values.

NameSeq: Ni=Ni+1:if Ni>i.max goto Terminate    'Reached end of main index

     gosub Show.Ni        'Display the main index
     FileNum=1:get#1,Ni+1 'Get namecode record [Rec#1 holds i.max & r.max]
     RecNum=cvi(n2$)      'Get main record number
     goto RefSeq1         'See if prospect fits target profile


     'THE `POSTCODE' ALPHABETIC SEQUENCER [SeqTyp=6]

     'This sequencing routine takes each prospect main record reference number
     'in turn in alphabetic order of postcode from the Postcode Index File and
     'passes it to the Selector routine which decides whether or not to place 
     'the prospect on the Target List.  

     'Note: Postcode index entry record numbers are biased one-up because
     'Record#1 is reserved for i.max and r.max values.

PostSeq: Pi=Pi+1:if Pi>i.max goto Terminate    'Reached end of main index

     gosub Show.Pi        'Display the main index
     FileNum=2:get#2,Pi+1 'Get ith postcode index entry
     Area$=left$(p1$,2)   'Get Postcode Area for Sales Area Selector routine
     RecNum=cvi(p2$)      'Get main record number
     goto RefSeq1         'See if prospect fits target profile


     'THE `RECORD NUMBER' NUMERIC SEQUENCER [SeqTyp=2]

     'This sequencing routine takes each prospect main record reference number
     'in turn [from 1 to r.max] and passes it to the Selector routine which 
     'decides whether or not to place the prospect on the Target List.  

RefSeq:  RecNum=RecNum+1:if RecNum>r.max goto Terminate   'end of main file

RefSeq1: gosub Show.RecNum:goto Select   'Form & display current main Record N
     '





     'THE ALPHA/GEOGRAPHIC SEQUENCER [SeqTyp=3,4,5]

     'GEOGRAPHIC GROUPER  This routine finds the upper and lower bounds of a
     'group of entries located in the same postcode zone. a postcode zone may
     'be a postcode area [SeqTyp=3], district [SeqTyp=4], or sector [SeqTyp=5]
     'according to which sequence type [SeqTyp] was specified by the user. 

     'GpTop  - upper bound of the current postcode zone
     'GpBase - lower bound of the current postcode zone
     'GpSize - number of prospects in the current geogroup built up by the
     '         AlphaSequencer from zero to the ultimate size of the GeoGroup.
     '         Once a GeoGroup is complete, the AlphaSequencer decrements
     '         GpSize as it finishes dealing with each member. AlphaSequencer
     '         displays GpSize as GpCnt, leaving the GpSize display intact.


     'BUILD THE FIRST/NEXT GEOGRAPHIC GROUP OF PROSPECTS

GeoGrouper:
     if Pi>i.max goto Terminate     'Have we hit the end of the index?
     GpSize=0                       'Start new GeoGroup
     GpBase=Pi:gosub Show.GpBase    'Set & display starting base for this scan
                                    '[Pi is initially =1]
GeoGrouper1:
     if Pi>i.max goto GeoGrouper3   'Have reached end of postcode index
                                    'so process final GeoGroup.
     FileNum=2:get#2,Pi+1           'Get Postcode and chop it down to area/
     x$=left$(p1$,2*(SeqTyp-2))     'district/sector precision as per SeqTyp.

     if Pi=GpBase then          _   'If first time through for this GeoGroup,
        GpPost$=x$                  'then set up the new `test postcode'.

     if x$=GpPost$ then             '[next] entry [still] in this GeoGroup
        GpSize=GpSize+1             'Add entry to current GeoGroup's size
        gosub Show.GpSize           'Display incremented group size
        Pi=Pi+1                     'Increment the postcode index
        gosub Show.Pi               'Display it
        goto GeoGrouper1            'See if next entry should be in GeoGroup
        end if


     'INITIALIZE THE GEO-GROUP SEARCH

GeoGrouper3:
     GpTop=Pi-1           'Highest postcode index entry in new GeoGroup
     gosub Show.GpTop     'Display it
     GpBase=Pi-GpSize     'Lowest  postcode index entry in new GeoGroup
     gosub Show.GpBase    'Display it
     GpCnt=GpSize         'Group `still to do' Count starts off = Group Size
     Ni=0                 'Initialize for alphabetic scan of namecode index
     '





     'STRUCTURE OF A GEO-GROUP                                Ŀ
     '                                              Pi=i.max  POSTCODERecNum
     'A GeoGroup is a range of entries                        Ĵ
     'in the Postcode Index which are                         POSTCODERecNum
     'in the same post zone:                                  Ĵ
     '                                                        POSTCODERecNum
     'SeqTyp 3 - Post zone = Post Area                 Ĵ
     'SeqTyp 4 -           = Post District       GpTop  Gi=5 POSTCODERecNum
     'SeqTyp 5 -           = Post Sector                     Ĵ
     '                                                  Gi=4 POSTCODERecNum
     '                                                       Ĵ
     '                                        GpSize     Gi=3 POSTCODERecNum
     '                                                       Ĵ
     '                                                  Gi=2 POSTCODERecNum
     '                                                       Ĵ
     'GpBase - i for first postcode index        GpBase Gi=1 POSTCODERecNum
     '         entry in current GeoGroup.             Ĵ
     '                                                        POSTCODERecNum
     '                                                        Ĵ
     '                                                   Pi=1 POSTCODERecNum
     '                                                        

     'GET [NEXT] PROSPECT NAMECODE FROM MAIN NAMECODE INDEX

AlphaSeq:
     gosub Show.GpCnt             'Display number of prospects left in GeoGroup
     if GpCnt=0 goto GeoGrouper   'Generate the [next] GeoGroup
AlphaSeq1:
     Ni=Ni+1:gosub Show.Ni        'Increment & display the main index
     FileNum=1:get#1,Ni+1         'Get the Namecode index entry
     RecNum=cvi(n2$)              'Get & display main record number from next
     gosub Show.RecNum            '[in alphabetic order] namecode index entry.


     'SEE IF HE IS A MEMBER OF THE CURRENT GEO-GROUP [IF SO DO ACCEPTANCE TEST]

     k=GpBase                      'Start from beginning of current geogroup
AlphaGeo:
     if k=Pi goto AlphaSeq1        'Reached end of current geogroup, so go back
     '                             'for another prospect from namecode index

     FileNum=2:get#2,k+1           'Get the postcode index entry
     Area$=left$(p1$,2)            'Post Area for Sales Area Selector routine

     if cvi(p2$)=RecNum then       'Compare record numbers
        GpCnt=GpCnt-1              'Dealt with one [more] group member
        goto Select                'See if prospect fits target profile
        end if
                                   'Current geogroup member is not the
     k=k+1:goto AlphaGeo           'prospect under test, so try next member
     '





     'REACHED THE END OF THE DATABASE - ALL PROSPECTS TESTED

Terminate:
     ExtractorState=5            'Advance the Main Sequence to termination
     RETURN                      'state and return to FMM polling loop.


     'MAIN SELECTION ROUTINE

     'This routine is called by whichever Sequencer Routine is active. It
     'effectively compares the current prospect's Market Profile with the
     '[normally broader] Target Profile. If the prospect's Market Profile
     'fits within the Target Profile, the prospect's main record number,
     'plus the elapsed times between his last and next events, are placed
     'in the next available record of the Target List file. It then returns
     'control to the Sequencer that called it.

Select: FileNum = 3 : get#3,RecNum    'Get the prospect's Key Values record
                                      'FORMAT        0000AAAA
                                      'SIC Number ٳ
                                      'Relation ٳ
                                      'Status ٳ
                                      'Last Event ٳ
                                      'Next Event 

     if mid$(k2$,2,1) < "A" then return 'Prospect record has been deleted.

     FileNum = 5 : get#5,RecNum         'Dates of prospect's Last & Next events


     'GET ELAPSED TIMES BETWEEN SYSTEM DATE & PROSPECT'S LAST+NEXT EVENT DATES

     dait$ = d1$ : gosub Elapsed2 : et1 = t : lapse1$ = lapse$
     dait$ = d2$ : gosub Elapsed2 : et2 = t : lapse2$ = lapse$

     goto  SicSelector                  'Go to first selection test
     '





     'SIC STRIP AND TRANSFER ROUTINE [Level 2 Subroutine]

     '         Ŀ                                        Ŀ
     'sic$(18) 1234       This routine strips any    sicn$(18) 1234    
     '         435        spaces off each element              435     
     '             8200   in the SIC array sic$()              8210    
     '         8210       and packs all non-null               56      
     '                    elements contiguously                7710    
     '         56         into the array sicn$().              8200    
     '             9      The elements of sicn$()              9       
     '                    are then used by the SIC                     
     '         7710       selector routine above to                    
     '            maximize execution speed.            


SicStrip: j=0                           'Index of validated SIC codes

     FOR k=1 TO 18                      'For each SIC field...
     x$=sic$(k)                         'get its contents
     l=len(x$)                          'and the number of characters in it
     y=0                                'Zeroise the while/wend escape switch
     WHILE l>0
     if y=0 then                        'If right-most valid chr not yet found
        if mid$(x$,l,1)>" " then y=l    'If this char is valid, note its posn.
        l=l-1                           'Move to next character left.
        else                            'If right-most valid char encountered
        if mid$(x$,l,1)<" " then        'then if this character is invalid
           y=0                          'set N valid chrs found to zero and
           l=0                          'set to escape from while/wend loop.
           else l=l-1                   'else move one character left.
           end if                       '
        end if                          '
     WEND                               '
     if y>0 then                        'If field contains valid characters
        j=j+1                           'Advance to next field in validated
        sicn$(j)=left$(x$,y)            'SIC numbers array, and store SIC N
        end if                          'set the escape switch, y.
     NEXT                               'Next unvalidated SIC field.

     if j<18 then                       'If less than 18 valid SIC Ns
        for k=j+1 to 18
        sicn$(k)="":next                'Clear the remaining fields.
        end if

     return                             'Return to Level 1
     '





     'SIC SELECTOR [Rejects if prospect's SIC No not found]

     'This routine first checks the SIC Status flag to see if there are
     'any SIC restrictions in this Target Profile. If there are none, it
     'returns immediately without rejecting the prospect.

     'If there are SIC restrictions, these are held in the Target Profile's
     '`sicn$()' array:

     'SIC ARRAY
     '9999 9999   There is space for upto 18 SIC numbers in the array. Each
     '9999 9999   number may be 1, 2, 3, or 4 digits long. (Trailing spaces, 
     '9999 9999   if present, are removed from each element of the `sicn()'
     '9999 9999   array at the very beginning of this program module. The
     '9999 9999   current prospect's SIC number is compared with each element
     '9999 9999   of the Target Profile's SIC array. Only as many digits of
     '9999 9999   the prospect's SIC number as there are digits in the SIC
     '9999 9999   element under test are actually compared. This allows for
     '9999 9999   variable-width SIC classifications to be specified in the
     '            Target Profile. Elements are compared with the prospect's
     'SIC number until either all 18 elements have been compared, or until
     'a null SIC element is encountered. If the prospect's SIC number [or the
     'appropriate number of digits thereof] matches one of the Target Profile's
     'SIC elements, then the prospect is accepted; otherwise he is rejected.

SicSelector:
     IF SIC.Flag>0 THEN                 'If SIC selection is in force, then
        j=0:y=0                         'Clear SIC element index & result flag
        WHILE y=0                       'While a final result not obtained
        j=j+1:y$=sicn$(j)               'Get [next] allowable SIC N
        if y$="" then                   'Null SIC N means no more SIC Ns to
           y=1                          'test, so set unsuccessful result.
    elseif y$=left$(k1$,len(y$)) then   'Required number of SIC digits match
           y=2                          'so prospect is accepted
    elseif j=18 then y=1                'If all 18 fields tried, so set for
           end if                       'unsuccessful result.
        WEND
        if y=1 goto Reject              'All SICs tested; prospect's not found
        END IF
     '





     'SEARCH FOR SALES AREA CAPTURE

     'This routine first checks the Sales Area Status flag to see if there are
     'any Sales Area (geographic) restrictions in this target profile. If
     'there are none, it returns control to the main selector routine without
     'rejecting the prospect.

     'It then loads the Sales Area status string from `key$(0)'. This is a
     'string upto 18 characters long. Each character [ranging A to R]
     'represents one of the upto 18 defined sales areas. The presence of a 
     'letter `A' for example means means that prospects in sales area N 1 may
     'be included in the Target List. A space indicates that the prospect is
     '(rejected). The sample below shows that if the prospect is in either the
     '2nd or the 5th sales area, he may be included; otherwise he is rejected:

     '[BE                ]

     'The 2-letter codes specifying the post areas which make up a given sales
     'area are held in a record#n of the file `AreaFile$' (opened as file #7
     'at the beginning of this program module) where `n' is the sales area's
     'number. When `got', the string of 2-character post area codes for sales
     'area `n' are contained in the file buffer string `a2$'.

     'The prospect's post area code [the first two characters of his postcode]
     'are then compared with each postarea in each `valid' sales area. If no
     'match is found, the prospect is rejected at this point; otherwise
     'control is returned to the main Selector Routine without rejecting him.

AreaSelector: if AreaFlag=0 then goto KeySelector 'No sales area restrictions

     if SeqTyp<3 then  
        FileNum=6:get#6,RecNum   
        Area$=left$(z1$,2)              'postarea from District File
        end if

     if Area$="  " then Area$="##"
     x$=key$(0):x=len(x$)               'Sales areas in this target profile
     j=0                                'For each sales area in this target profile...

AreaSelect1:
     j=j+1:if j>x goto Reject           'Prospect's postcode not found
     FileNum=7'
     get#7,asc(mid$(x$,j,1))-64         'Get [next] sales area's post area codes
     sa=cvi(a1$)                        'Number of post areas in this sales area
     k=0                                'For each post area in this sales area..

AreaSelect2:
     k=k+1:if k>sa goto AreaSelect1     'Prospect's postcode not in this area
     if Area$=mid$(a2$,k+k-1,2)      _  'Prospect's postcode has been found
        goto KeySelector                'so do not reject this prospect
     goto AreaSelect2                   'Try next postcode in this sales area
     '





     'KEY SELECTOR [Rejects prospect if one of his key values don't match]

     '         TargStat$   [  **  **  ]
     '                        ||  ||
     '         relation ------'|  ||        Asterisk if Target Profile
     '         status ---------'  ||        sets any restrictions
     '         last event --------'|        on the respective key.
     '         next event ---------'

     '                        k2$  ABCD
     '                             ||||
     'Prospect's  relation---------'|||     The value (A-R) of the
     '            status  ----------'||     prospect's value of 
     '            last event---------'|     the respective key.
     '            next event----------'

     'key$(0)
     'key$(1)  ABCDEFGHIJKLMNOPQR    Target's permitted Relation values
     'key$(2)  ABCDEFGHIJKLMNOPQR    Target's permitted Status values
     'key$(3)                     
     'key$(4)  ABCDEFGHIJKLMNOPQR    Target's permitted Last Event types
     'key$(5)  ABCDEFGHIJKLMNOPQR    Target's permitted Next Event types


     'SEE IF PROSPECTS RELATION KEY IS SET TO ONE OF THE TARGET'S VALUES

KeySelector:
     if Rel.Flag>0 then
        if instr(key$(1),mid$(k2$,1,1))=0 goto Reject
        end if

     'SEE IF PROSPECTS STATUS KEY IS SET TO ONE OF THE TARGET'S VALUES

     if StatFlag>0 then
        if instr(key$(2),mid$(k2$,2,1))=0 goto Reject
        end if

     'SEE IF PROSPECTS LAST EVENT KEY IS SET TO ONE OF THE TARGET'S VALUES

     if LastFlag>0 then
        if instr(key$(4),mid$(k2$,3,1))=0 goto Reject
        end if

     'SEE IF PROSPECTS NEXT EVENT KEY IS SET TO ONE OF THE TARGET'S VALUES

     if NextFlag>0 then
        if instr(key$(5),mid$(k2$,4,1))=0 goto Reject
        end if
     '





     'ATTRIBUTE SELECTOR [Sets reject flag if no matching attributes]

     'This routine first checks the Attribute Status flag to see if there are
     'any Attribute (product line) restrictions in this target profile. If
     'there are none, it returns control to the main selector routine without
     'rejecting the prospect.

     'aa1$    ABCDEFGHIJKLMNOP     Prospect can have upto 16 attribute values
     'key$(3) ABCDEFGHIJKLMNOPQR   Target's permitted attribute values

     'The routine interprets each capital letter [A to R] representing the
     'attribute values held by the prospect as an interger 1 to 18. This is
     'then used to `look' in the corresponding position along the Target's
     '`permitted attributes' string to check to see if there is an asterisk
     'in that position. If there is, then control is returned immediately to
     'the main Selector routine. If not, the prospect is NOT immediately 
     'rejected; but the rest of the prospect's attributes are checked to
     'see if he has any in common with the Target's attribute pattern. This
     'in effect means that if the prospect concerned has an interest in ANY
     'product line specified in the Target Profile, then he is INcluded in
     'the Target List. He is only rejected if he has NO attributes in
     'common with the Target Profile.


AttrSelector: if AttrFlag=0 goto CodeSelector  'No restriction

     FileNum=4:get#4,RecNum               'Get prospect's attribute selections
     x$=key$(3)                           'Get attributes target selections
     j=0                                  'Start with first attribute selection

AttrSelect:   j=j+1:if j>15 goto Reject

     '        [ABCDEFGHIJKLMNOP] Prospect's attributes
     '                    from his attribute record
     z$=mid$(aa1$,j,1)
     '         
     '        letter representing prospect's jth attribute

     if z$<"A" or z$>"R" goto Reject

     if instr(x$,z$)>0 goto CodeSelector  'Accepted
     '                      
     '          letter representing prospect's jth attribute
     '        
     '        [ABCDEFGHIJKLMNOPQR] Target's `permitted' attributes

     goto AttrSelect   'Try the next one
     '





     'CODES STRIP AND TRANSFER ROUTINE [Level 2 Subroutine]

     'Cde$(18)
     'Ŀ                            
     '                                   This routine strips any
     'AAA997/nn55*****                   spaces off each element
     '                                   in codes array Cde$() 
     '                XXB884/aa42*****   and packs all non-null 
     '                                   elements contiguously  
     '                                   into the array code$().
     'AAA975/hh42*****                   The elements of code$()
     '                                   are then used by the code
     '                                   selector routine below to
     '   maximize execution speed.

     'code$(18)
     'Ŀ                            
     'AAA997/nn55*****                   Note that any character => " "
     'AAA975/hh42*****                   may be used as a selector
     'XXB884/aa42*****                   except for the asterisk which 
     '                                   is the `wild charcter'.
     '                                   Positions occupied by an 
     '                                   asterisk are ignored as far
     '                                   as the selection process is
     '                                   concerned. Any element containing
     '                                   a character>" " will be used as a
     '   selection string.

CodeStrip:
     if CodeFlag=0 then return          'Code selection not active
     NumCodes=0                         'Number of valid code lines
     for k=1 to 18                      'For each code line in the array of 18
     x$=Cde$(k)                         'Get the code line
     if x$ > space$(16) then            'If it contains more than just spaces,
        l=len(x$)                       'length [N of characters]
        while l>0
        if mid$(x$,l,1) < " " then      'Chop off right-most invalid characters
           l=l-1                        '
      else NumCodes=NumCodes+1          'Some valid code characters in this line
           code$(NumCodes)=left$(x$,l)  'so increment output element N & save
           l=0
           end if
        wend
        end if
     next                               'next code line
     return                             'Return to Level 1
     '





     'CODE SELECTOR

     'This routine first checks the Code Status flag to see if there are
     'any Selection Code restrictions in this target profile. If not, it
     'returns control to main selector routine without rejecting the prospect.

     ' Selection Code Array cde$(18)
     ' xxxx**xx**xxxxxx  xxxxxxxxxxxxxxxx   Each selection code array element
     ' xxxxxxxxxxxxxxxx  xxxxxxxxxxxxxxxx   contains upto 16 ascii characters.
     ' xxxxxxxxxxxxxxxx  xxxxxxxxxxxxxxxx   Each may be from null to chr$(255)
     ' xxxxxxxxxxxxxxxx  xxxxxxxxxxxxxxxx   The asterisk `*' is the wild chr.
     ' xxxxxxxxxxxxxxxx  xxxxxxxxxxxxxxxx   it marks a character position in
     ' xxxxxxxxxxxxxxxx  xxxxxxxxxxxxxxxx   an element which is to be ignored
     ' xxxxxxxxxxxxxxxx  xxxxxxxxxxxxxxxx   by this selection routine. 
     ' xxxxxxxxxxxxxxxx  xxxxxxxxxxxxxxxx
     ' xxxxxxxxxxxxxxxx  xxxxxxxxxxxxxxxx   Each prospect has a single 16 byte
     '                                      selection code kept in his record
     'within the CodeFile$. This routine compares each character of the pros-
     'pect's selection code with each element of the Selection Code array
     'except characters which co-incide with asterisks. If the characters in
     'the prospect's selection code coincide completely with the character
     'pattern in any one of the Selection Code array elements [ignoring
     'positions in each element which contain asterisks], then the prospect
     'is not rejected by this routine from the Target List. If no match is
     'found with any of the 18 array elements, then he is rejected.

CodeSelector:
     if CodeFlag=0 goto TimeSelector    'No restriction
     FileNum=8:get#8,RecNum:j=1         'Get this prospect's code pattern
CodeSelect1:
     y$=code$(j):k=1                    'Get [next] target code pattern
CodeSelect2:                            'Isolate [next] character in prospect
     z$=mid$(y$,k,1)                    'code pattern
     if z$="*" goto CodeSelect3         '* is the wild card character
     if z$<>mid$(c1$,k,1) _
        goto CodeSelect4                'A character mis-match, so reject

CodeSelect3:
     if k<16 then _
        k=k+1:goto CodeSelect2          'Try the next character for a match
     goto TimeSelector                  'All required characters match

CodeSelect4:
     if j<NumCodes then _
        j=j+1:goto CodeSelect1          'Try the next target code pattern
     goto Reject                        '






     'ELAPSED TIME SELECTOR

     'The elapsed time between the prospect's last event and today, `et1' and
     'the elapsed time between today and the prospect's next event, `et2',
     'were calculated unconditionally for this prospect at the beginning of
     'the Selector Routine. The Target Profile `time limitations' array con-
     'tains the following information:

     'tme(1)  Last Event:  Early time limit                 T1
     'tme(2)               Late time limit                  T2
     'tme(3)  Next Event:  Early time limit (past)          T1
     'tme(4)               Late time limit (future)         T2

     'tme(5)  Last Event:  Early time limit in effect flag  T1 valid
     'tme(6)               Late time limit in effect flag   T2 valid
     'tme(7)  Next Event:  Early time limit in effect flag  T1 valid
     'tme(8)               Late time limit in effect flag   T2 valid

     'This subroutine uses this information together with `et1' and `et2' to
     'determine whether or not this Prospect's `last event' and `next event'
     'fall within the `time window' specified by the `tme()' array. If either
     'event falls outside the window (when the event's `limit in effect' flag
     'is set) then the prospect is rejected from the Target List.


     'MAIN SUBROUTINE [FIRST LEVEL SUBROUTINE]

TimeSelector:

     if LDteFlag>0 then           'If a `Last Event' restriction, then
        if tme(5)>0 and    _      'If an early limit restriction 
           et1>tme(1)      _      'which is beyond early time limit or
        or tme(6)>0 and    _      'If a restriction on late time limit
           et1<tme(2)      _      'which is beyond late time limit
        or lapse1$="////"  _      'or elapsed time invalid, then
           goto Reject            'reject this prospect.
        end if

     if NDteFlag>0 then           'If a `Next Event' restriction
        if tme(7)>0 and    _      'If a restriction on early limit
           et2>tme(3)      _      'which is beyond early time limit, or
        or tme(8)>0 and    _      'If a restriction on late time limit
           et2<tme(4)      _      'which is beyond late time limit
        or lapse2$="////"  _      'or elapsed time invalid, then
           goto Reject            'reject this prospect.
        end if
     '





     'THIS PROSPECT FITS THE TARGET PROFILE

     if Exceptions=1 goto Reject1
Accept1:
     accepts=accepts+1           'Increment number of accepted prospects
     gosub Show.accepts          'Display the number accepted
     rset s1$=mki$(RecNum)       'Accepted Prospect's Record Number
     rset s2$=mki$(et1)          'His elapsed time since last event
     rset s3$=mki$(et2)          'His elapsed time since next event
     FileNum=9:put#9,accepts+1   'Store accepted prospect on Target List
     RETURN                      'Return to the FMM polling loop


     'THIS PROSPECT DOES NOT FIT THE TARGET PROFILE

Reject:
     if Exceptions=1 goto Accept1
Reject1:
     rejects=rejects+1           'Increment the number of rejected prospects
     gosub Show.rejects          'Display it on the screen monitor panel
     RETURN                      'Return to the FMM polling loop
     '





     'DISPLAY EXTRACTION PROCESS MONITOR

ShowExtrBox:
     Column=1:colour=19:ps$=space$(40):row= 5:gosub 50
     ps$="  EXTRACTING TARGET LIST OF PROSPECTS   ":row= 6:gosub 50
     ps$="                                        ":row= 7:gosub 50
     ps$=" Namecode Index        Sequencing       ":row= 8:gosub 50
     ps$=" Postcode Index        Index Size       ":row= 9:gosub 50
     ps$="                                        ":row=10:gosub 50
     ps$=" N in Group           Max extent       ":row=11:gosub 50
     ps$=" N unsorted           of records       ":row=12:gosub 50
     ps$="                       file......       ":row=13:gosub 50
     ps$=" Group Top                              ":row=14:gosub 50
     ps$=" Group Base                             ":row=15:gosub 50
     ps$="                       Accepted:        ":row=16:gosub 50
     ps$=" Record Number         Rejected:        ":row=17:gosub 50
     colour=1                                    
     ps$=string$(40,223):row=18:gosub 50
     ps$=space$(40):for row=19 to 24:gosub 50:next

     ps$="EXTRACTING TARGET LIST OF PROSPECTS":row=6:column=3:colour=27:gosub 50
     row=23:column=11:colour= 2:ps$="Press Esc to Abort":gosub 50
            column=17:colour=10:ps$="Esc"               :gosub 50
     locate 20,1:return

ShowMax: colour=23
     ps$=str$(SeqTyp):row= 8:gosub ShowMax1
     ps$=str$(i.max) :row= 9:gosub ShowMax1
     ps$=str$(r.max) :row=13:gosub ShowMax1
     return
ShowMax1: column=40-len(ps$):goto 50

Fin:    ps$=" FINISHED " :row= 6:column=3:colour=10:gosub 50:return
ErrFin: ps$=" FAILED!  " :row= 6:column=3:colour=12:gosub 50:return
     '





     'PARAMETER DISPLAY SUBROUTINES

Show.Ni    : row= 8:ps$=str$(Ni)    :goto ShowNum       'Namecode Index
Show.Pi    : row= 9:ps$=str$(Pi)    :goto ShowNum       'Postcode Index

Show.GpSize: row=11:ps$=str$(GpSize):goto ShowNum       'Group Size
Show.GpCnt : row=12:ps$=str$(GpCnt ):goto ShowNum       'GeoGroup Index

Show.GpTop : row=14:ps$=str$(GpTop ):goto ShowNum       'Group Index
Show.GpBase: row=15:ps$=str$(GpBase):goto ShowNum       'Group Base

Show.RecNum: if RecNum>0 and RecNum<=r.max then
             row=17:ps$=str$(RecNum):goto ShowNum       'Record Number
             end if
             column=46:colour=79
             ps$="Ŀ":row= 9:gosub 50
             ps$="  BAD RECORD N IN AN INDEX  ":row=10:gosub 50
             ps$="Press Esc key, then go to the":row=11:gosub 50
             ps$="Configuration Option [9] of  ":row=12:gosub 50
             ps$="the Main Menu and run a re-  ":row=13:gosub 50
             ps$="construction of the database.":row=14:gosub 50
             ps$="":row=15:gosub 50
             return RecNumError         'Return from level 2 to level 1 subr

Show.accepts: row=16:ps$=str$(accepts):goto ShowNum1  'No of records accepted
Show.rejects: row=17:ps$=str$(rejects):goto ShowNum1  'No of records rejected
ShowNum:  column=16:goto ShowNum2
ShowNum1: column=34
ShowNum2: colour=23:ps$=left$(ps$,6):ps$=space$(6-len(ps$))+ps$:gosub 50:RETURN
     '





     'DISPLAY LOCKED FILES INDICATOR

FileIndicator:
     Column=41:colour=grey
     ps$="                                        ":row=19:gosub 50
     ps$="     Open files:  NAMECODE DATES      ":row=20:gosub 50
     ps$="                  POSTCODE DISTRICT   ":row=21:gosub 50
     ps$="                  KEYS     AREAS      ":row=22:gosub 50
     ps$="     TARGET LIST ATTRIBS  CODES      ":row=23:gosub 50
     ps$="                                        ":row=24:gosub 50
     return

FileLamps:  ps$=""
            k=1:row=20:column=59:gosub FileLamps1
            k=2:row=21          :gosub FileLamps1
            k=3:row=22          :gosub FileLamps1
            k=4:row=23          :gosub FileLamps1
            k=5:row=20:column=69:gosub FileLamps1
            k=6:row=21          :gosub FileLamps1
            k=7:row=22          :gosub FileLamps1
            k=8:row=23          :gosub FileLamps1
            k=9:row=23:column=46:gosub FileLamps1
            return

FileLamps1:
     if FilesOpen=0 then                'If it is the first time through, then
        if FileReqd(k)=1 then           'If this file is to be opened, then
           colour=6                     'light its lamp in dull amber,
      else colour=8                     'show it as off (grey)
           end if                       '
        else                            'Otherwise, [if not the first pass]
        if FileOpen(k)=1 then           'Iif the file is now open, then
           colour=10                    'light its lamp bright green,
      else return                       'else do nothing and exit
           end if                       '
        end if                          '

     goto 50                            'Display the lamp and return
     '





     'DECIDE WHICH FILES NEED TO BE OPENED FOR THIS EXTRATION

OpeningCriteria:
                      FileReqd(1)=1  'Namecodes must always be opened initially
     if SeqTyp>2 then FileReqd(2)=1  'Postcodes needed for all except SeqTyp 1.
                      FileReqd(3)=1  'KeysFile
                      FileReqd(5)=1  'DateFile
     if SeqTyp<3 then FileReqd(6)=1  'DistFile
                      FileReqd(9)=1  'Target List


     'OPEN THE NECESSARY KEY FILES

     '    SIC             An asterisk present means that the key
     '    sales area      concerned imposes a restriction on the
     '    relation        Target Profile, and that therefore the
     '    status          appropriate files for handling that key
     '    products        selection have to be opened.
     '    selector code
     '    last event type     This routine opens only those files
     '    next event type     containing information required by
     '    last event date     those selector routines dealing with
     '    next event date     keys which impose restrictions in
     '                          the current Target Profile.
     '  "**********" TargStat$

     x$="    *     ":gosub Need:if x<0 then FileReqd(4)=1  'AttrFile
     x$=" *        ":gosub Need:if x<0 then FileReqd(7)=1  'AreaFile
     x$="     *    ":gosub Need:if x<0 then FileReqd(8)=1  'CodeFile

     FilesOpen=0:goto FileLamps         'Display lamps in dim amber

Need:x=10:While x>0                     'Level 3 subroutine

     if mid$(TargStat$,x,1)="*" and _   '
        mid$(x$,x,1)       ="*" then    '
        x=-1                            'Set to open the file
        end if                          '

     x=x-1:wend:return                  '
     '





     'UNSCRAMBLE THE TARGET STATUS WORD INTO INDIVIDUAL FLAGS

FlagSub:
     k= 1:gosub FlagSub1:SIC.Flag=x
     k= 2:gosub FlagSub1:AreaFlag=x
     k= 3:gosub FlagSub1:Rel.Flag=x
     k= 4:gosub FlagSub1:StatFlag=x
     k= 5:gosub FlagSub1:AttrFlag=x
     k= 6:gosub FlagSub1:CodeFlag=x
     k= 7:gosub FlagSub1:LastFlag=x
     k= 8:gosub FlagSub1:NextFlag=x
     k= 9:gosub FlagSub1:LDteFlag=x
     k=10:gosub FlagSub1:NDteFlag=x
     return

FlagSub1: if mid$(targstat$,k,1)="*" then x=1 else x=0
          return


     'ERROR MESSAGES DISPLAY ROUTINE

ShowMessages:
     if ShowMessages>0 then             'If there is a message to be displayed
        ShowMessages=0                  'then cancel the display request

        if DosMsg$="WAITING FOR ACCESS TO FILE" then
           DosMsg$=DosMsg$+" "+FileName$(FileNum)
           end if

        ps$=left$(DosMsg$,DosLen)       'Chop or pad the message to the length
        ps$=ps$+space$(DosLen-len(ps$)) 'of the message display field.
        row=DosRow:column=DosCol        'Set the screen display co-ordinates
        colour=DosClr:gosub 50:DosClr=0 'Display the message & cancel colour.
        end if

9999 return                             'Return to the FMM polling loop.

