





     '-------------------------------------------------------------------------
     'FILE OPENING ERROR HANDLER           12FEB87         [File: OpenFile.bas]
     '-------------------------------------------------------------------------
     '
     'Author : R J Morton
     '
     'This routine attempts to open a disk file as file number FileNum and
     'then return with the named file open and locked. If an error occurs
     'during the attempt to open the file, the routine first checks the
     'MS-DOS error code to see if the error is one likely to have resulted from
     'a file being already in use by another PC on the network. If it is, then
     'the routine waits 3 to 4 seconds and then tries again to open the file.
     'It retries a specifiable number of times before giving up and regarding
     'the error as fatal. If the error is a fatal type anyway, it displays the
     'MS-DOS error message in bright red at the screen position specified by
     'ErrMsgRow and ErrMsgCol and then returns to the calling program. When a
     'possible temporary access error occurs, it displays the error message in
     'bright amber while retrying.
     '
     'Inputs:  Retries   - Number of retries permitted for opening the file
     '                     in the event of a file access error.
     '         FileName$ - Filespec [path+file name] of file to be opened
     '         FileNum   - number under which the file is to be opened
     '         RecLen    - if > 0 it means record length for the above file 
     '                     if = 0 open a file for input
     '                     if =-1 open a file for output
     '                     if =-2 open a communications port for I/O
     '         ErrMsgRow - screen co-ordinates for error message display
     '         ErrMsgCol - [These default to row 3, column 45 if unspecified]
     '
     'Locals:  ErrNum  - preserving variable for the MS-DOS error number
     '         NumTries- number of remaining attempts to open the file.
     '         Timeout - number of seconds left to run before a re-try
     '         Tyme$   - reference string to capture & hold system time
     '
     '-------------------------------------------------------------------------
     'OPEN THE REQUIRED FILE
     '
OpenFile:
4100 if ErrMsgRow=0 then ErrMsgRow= 3       'Default error message
     if ErrMsgCol=0 then ErrMsgCol=45       'co-ordinates
     '
     if retries=0 then retries=4
     NumTries=Retries                       'Initialize `re-tries' counter
     '
OpenFile0:
     ErrNum=0                               'Clear local Error N variable
     '
     if FileName$="" then ps$="Null filename error":goto OpenFile6
     if FileNum<1 or FileNum>20 then ps$="File N out of range":goto OpenFile6
     '
     '-------------------------------------------------------------------------
     '






     '-------------------------------------------------------------------------
     '
     on error goto OpenFile3       
     '
     if RecLen>2048 then ps$="Specified record length > 2048.":goto OpenFile6
     '
     if RecLen> 1 then open FileName$ as FileNum Len=RecLen:goto OpenFile1
     if RecLen= 0 then open FileName$ for  input as FileNum:goto OpenFile1
     if RecLen=-1 then open FileName$ for output as FileNum:goto OpenFile1
     if RecLen=-2 then open FileName$ as FileNum           :goto OpenFile1
     '
     ps$="File type not specified: abort.":goto OpenFile6
     '
     '-------------------------------------------------------------------------
     'TRAP ERROR CODES WHICH MAY HAVE RESULTED FROM NETWORK DELAYS
     '
OpenFile3:
     ErrNum=err            'Preserve the MS-DOS error number
     resume OpenFile4      'Reset error DOS condition
     '
OpenFile4:
     if    ErrNum=53 _        'File not found
        or ErrNum=55 _        'File already open
        or ErrNum=70 _        'Permission denied
        or ErrNum=71 _        'Disk not ready
        or ErrNum=75 _        'Path not found
        or ErrNum=76 _        'Path/File access error
           goto OpenFile8              
     '
     '-------------------------------------------------------------------------
     'DISPLAY ERROR AND MANUAL INERVENTION PROMPT
     '
OpenFile5:
     ps$=er$(ErrNum)                                  'Display error message
     '
OpenFile6:
     if ErrNum=0 then ErrNum=99                       'Non-DOS error
     colour=12:gosub OpenFile11
     ps$="Press    to re-try or Esc to abort:"        'Display user prompt
     gosub OpenFile12                                 'in green.
     '
     colour=10
     column=ErrMsgCol+ 6:ps$=chr$(17)+"":gosub 50    'Prompt highlights
     column=ErrMsgCol+22:ps$="Esc"       :gosub 50    'in bright green.
     '
     '-------------------------------------------------------------------------
     '






     '-------------------------------------------------------------------------
     'AWAIT MANUAL KEYBOARD INTERVENTION
     '
     sound 3300,3                      'Error tone
     gosub ClrKbdBuf                   'Clear the keyboard input buffer
     locate ErrMsgRow+1,ErrMsgCol+35   'Set cursor position
OpenFile7:
     gosub InChar                      'Await an incoming character
     if from>0 goto OpenFile7          'Ignore non-keyboard input
     '
     ps$="":gosub OpenFile12           'Wipe prompt
     '
     if x=13 goto OpenFile0            'Manual instruction to try again
     if x=27 goto OpenFile1            'Abort the attempt to open the file
     beep:   goto OpenFile5            'Bum keystroke
     '
     '-------------------------------------------------------------------------
     'DECREMENT N OF RE-TRIES, SHOW MESSAGE, DELAY 4 SECS, THEN RETRY
     '
OpenFile8:
     if ErrNum=55 then close FileNum 'File already open error
     if NumTries=0 goto OpenFile5    'Fatal error: manual intervention
     NumTries=NumTries-1
     '
     ps$="Network access delay: re-trying.":colour=14:gosub OpenFile11
     '
     timeout=4       'Set timeout for a 3 to 4 second delay
OpenFile9:
     tyme$=time$     'Set time variable = system time
OpenFile10:
     if tyme$=time$ _               'Wait until system time changes
        then goto OpenFile10        '[which it does every second]
     '
     timeout=timeout-1              'Decrement the `seconds left' counter
     if timeout>0 goto OpenFile9    'Wait another second
     goto OpenFile0                 'Go and re-try to the file
     '
     '-------------------------------------------------------------------------
     'ERROR MESSAGE DISPLAY SUBROUTINE
     '
OpenFile11: row=ErrMsgRow  :goto OpenFile13    'Error message
OpenFile12: row=ErrMsgRow+1:colour=2           'User prompt
OpenFile13: ps$=ps$+space$(36-len(ps$))        'Pad message to 36 characters
            column=ErrMsgCol:gosub 50
            return
     '
     '-------------------------------------------------------------------------
     'EXIT ERROR HANDLER
     '
OpenFile1: ps$="":colour=7:gosub OpenFile11    'Wipe error message
OpenFile2: on error goto 0            'Exit error handler
4199 x$="":x=0:return                 'Clear keyboard character
     '
     '-------------------------------------------------------------------------
     '
