






     '-------------------------------------------------------------------------
     'PRINTER I/O CONTROL          28DEC86                  [File: PrintIO.Bas]
     '-------------------------------------------------------------------------
     '
     'Author : R J Morton
     '
     'Handles the printing of a disk file PrtSpool$ [LastLine=0] or an array
     'PrintArray$(nn) [LastLine=number of lines to be printed from the array]
     'to a parallel or serial port. When printing from a file, it waits for a
     'keystroke before starting to print. When printing from an array, it 
     'starts automatically.
     '
     'Inputs PRTSPOOL$ - Name of the file containing the text to be printed
     '          PAPER  - 1 standard 15 by 11 inch listing paper
     '                   2 special sticky label stationery
     '                   3 headed company A4 notepaper
     '                   4 plain A4 typing paper
     '          PRTROW - First row of the printer display panel
     '          PRTCOL - First column of the printer display panel
     '            CRLF - 1 send line feeds after CRs as given; 0 don't
     '        LastLine - N of lines in print array
     '        DontWait - if =1, dont wait for keystroke to start printing
     '
     Dim PrintArray$(70) 'Source array for print output
                         'when not using the spool file
     '
     'Locals  COMM   - 0 printer connected to a parallel port
     '                 1 printer connected to COM1
     '                 2 printer connected to COM2
     '       PRTLAMP - Number of printer status lamp to be lit
     '
     '-------------------------------------------------------------------------
     '






     '-------------------------------------------------------------------------
     'INITIALIZATION
     '
PrintIO:
     PrtLamp= 0                       'Clear the printer status lamps
     gosub ShowPpanel                 'Display printer control panel
     gosub ShowPreStartMsgs           'Display pre-start messages
     locate PrtRow+1,PrtCol+29        'Show cursor in print control box
     '
     if DontWait>0 goto PrintIO0      'Print from the array
     '
     gosub ClrKbdBuf                  'Clear keyboard buffer
     gosub InChar                     'Wait for keystroke to start printout
     if x=27 then abort=1:return      'Exit without doing anything
     '
PrintIO0: abort=0:FinishedPrinting=0  'Clear the abort & finished flags
     '
     if left$(printer$,3)="COM" then
        comm=val(mid$(printer$,4,1))    'COM channel N 1 or 2
        if comm<1 or comm>2 then comm=0 'Determine printer channel
 elseif left$(printer$,3)="LPT" then
        lpt=val(mid$(printer$,4,1))     'COM channel N 1 or 2
        if lpt<0 or lpt>3 then lpt=0
        if right$(printer$,2)="LF" then
           crlf=1
           printer$=left$(printer$,5)
      else crlf=0
           end if
        end if
     '-------------------------------------------------------------------------
     'TRY TO OPEN THE PRINTER CHANNEL
     '
PrintIO2:
     on error goto PrintIO18              'In case printer won't open
     open printer$ as #1                  'Open printer channel
     gosub PrintIO24                      'Delay before sending first chr
     width#1,255                          'No auto C/R at any line width
     print#1,PrinterInit$;                'Printer initialisation string
     '
     PrtLamp= 1                           'Clear the printer status lamps
     gosub ShowPLamps                     'Show print lamp
     gosub ShowPostStartMsgs              'Display pre-start messages
     gosub ClrKbdBuf                      'Clear keyboard input buffer
     '
     if LastLine=0                          _  'If printing from spool file,
        then open PrtSpool$ for input as #2 _  'then open spool file
        else gosub PrintIO21                   'else initialize array input
     '
     '-------------------------------------------------------------------------
     '





     '-------------------------------------------------------------------------
     'TEST FOR KEYBOARD INTERVENTION AND PRINTER X-OFF
     '
PrintIO3:
     '
     shot=1:gosub InChar                'Test for keystroke
     if x>0 goto PrintIO9               'Goto keyboard interrupt handler
     if comm>0 goto PrintIO14           'Serial printer, so check for Xoff
     '
     '-------------------------------------------------------------------------
     'TEST FOR END OF FILE OR END OF ARRAY
     '
PrintIO4:
     if LastLine>0 goto PrintIO5        'Source is spool array
     if eof(2) goto PrintIO8            'End of spool file, so exit
     '
     '-------------------------------------------------------------------------
     'GET THE NEXT CHARACTER FROM THE APPROPRIATE SOURCE
     '
PrintIO5:
     '
     if LastLine=0           _          'Number of lines in spool array
        then x$=input$(1,#2) _          'Get character from spool file
        else gosub PrintIO22            'Get character from spool array
     '
     '-------------------------------------------------------------------------
     'CHECK & EXECUTE LF-AFETR-CR REQUIREMENT
     '
     if x$=chr$(13) then
        if PageDepth>0 then
           LineCount=LineCount+1
           if LineCount>PageDepth then print#1,chr$(12);:LineCount=0
           end if
        if crlf$=chr$(10) then y$="           " else y$=""
        end if
     if crlf=1 goto PrintIO6            'Need to send line feeds
     if x$<>chr$(10) goto PrintIO6      'Not a line feed character
     '
     if crlf$=chr$(13) _
        then crlf$=x$  _
       :goto PrintIO4                   'It was preceeded by a C/R, so skip
     '
     '-------------------------------------------------------------------------
     'SEND THE CHARACTER TO THE PRINTER
     '
PrintIO6: print#1,y$x$;:crlf$=x$:goto PrintIO3
     '
     '-------------------------------------------------------------------------
     '






     '-------------------------------------------------------------------------
     'EXIT SEQUENCE
     '
PrintIO7:
     abort=1                              'Exit for aborted printout
     print#1,chr$(10)                     'Send final line feed
     print#1,"Printout aborted!"
PrintIO8:
     print#1,chr$(12);                    'Thow the page
     tyme$=time$:while tyme$=time$:wend   'One second delay.
     on error goto 0                      'Release the error trap
     close                                'Close print file & printer channel
     PrtLamp=0:gosub ShowPLamps           'Clear `printing' indicator lamp
     return                               'Return to calling program
     '
     '-------------------------------------------------------------------------
     'KEYBOARD INTERRUPT ROUTINE
     '
PrintIO9:
     gosub ClrKbdBuf                      'Clear keyboard buffer
     if x=13 goto PrintIO10               'Print halt request
     if x=27 goto PrintIO7                'Abort while priting
             goto PrintIO3                'Bum character
     '
     '-------------------------------------------------------------------------
     'TEMPORARY HALT ROUTINE
     '
PrintIO10:
     '
     halt=1   :gosub ShowPostStartMsgs    'Display resume message
     PrtLamp=2:gosub ShowPlamps           'Light the HALT lamp
     '
PrintIO11:
     '
     gosub ClrKbdBuf                      'Clear keyboard buffer
     gosub InChar                         'Wait for character from keyboard
     if x=27 goto PrintIO7                'Abort while halted
     if x=13 goto PrintIO13               'Resume
             goto PrintIO11               'Bum character
     '
     '-------------------------------------------------------------------------
     'RESUME PRINTING
     '
PrintIO13:
     halt=0   :gosub ShowPostStartMsgs    'Display temporary halt message
     PrtLamp=1:gosub ShowPlamps           'Light the PRINT lamp
     goto PrintIO3                        'Go get & print next character
     '
     '-------------------------------------------------------------------------
     '






     '-------------------------------------------------------------------------
     'X-OFF X-ON HANDLER
     '
PrintIO14:
     if loc(1)=0 goto PrintIO4    'See if there is input from printer
     x$=input$(1,1)
     if x$="" then x=0 else x=asc(x$)
     if x<>19 goto PrintIO5                 'Ignore anything other than Xoff
     PrtLamp=2:gosub ShowPlamps             'Light the Xoff lamp
     '
PrintIO15:
     if loc(1)=0 goto PrintIO14   'See if there is input from printer
     x$=input$(1,1)
     if x$="" then x=0 else x=asc(x$)
     if x<>17 goto PrintIO16                'Ignore anything other than Xon
     PrtLamp=1:gosub ShowPlamps             'Light the PRINT lamp
     goto PrintIO4
     '
PrintIO16:
     shot=1:gosub InChar         'Singleshot through INCHAR
     if x=0 goto PrintIO15       'No input from keyboard
     if x= 0 goto PrintIO15      'No keystroke
     if x=27 goto PrintIO7       'Keyboard forced abort
     if x=32 goto PrintIO4       'Keyboard forced Xon
     goto PrintIO15              'Wait for Xon
     '
     '-------------------------------------------------------------------------
     'DEVICE TIME-OUT ERROR HANDLER
     '
PrintIO18:
     PrtErr=err:resume PrintIO19          'Get out of the error condition
PrintIO19:
     PrtLamp=4:gosub ShowPlamps           'Light the ERROR lamp
     halt=1   :gosub ShowPostStartMsgs    'Display the RESUME message
     on error goto 0:close                'Close the printer
PrintIO20:
     gosub InChar:if x=0 goto PrintIO20   'Wait for keystroke
     halt=0
     if x=27 then close:abort=1:return    'Abort after error
     goto PrintIO2                        'Try to re-open the printer channel
     '
     '-------------------------------------------------------------------------
     '






     '-------------------------------------------------------------------------
     'PRINT FROM AN ARRAY
     '
     'As an alternative to printing from a spool file `PrintIO.Bas' also allows
     'you to print from an array called `PrintArray$(nn)' which must be dimens-
     'ioned in your main program. To switch the source frm the spool file to
     'the array, you must set the ArrayFlag to the number of lines (elements)
     'in the array. If this is zero, `PrintIO.Bas' assumes that input is to be
     'taken from the spool file `PrtSpool$'.
     '
     '-------------------------------------------------------------------------
     'INITIALIZATION
     '
PrintIO21:
     '
     if LastLine=0 _  'N of lines in print array set before entering PrintIO
        then return   'If it is zero, then print from PrtSpool$ file
     '
     PrintPointer=0   'Set character-within-line pointer before first character
     PrintLine=-1     'Set print-line-N to start with line N 0 of print array
                      'PrintArray$(0) contains printer control codes
     LineLength=0     'Trigger to get first line of print
     return
     '
     '-------------------------------------------------------------------------
     'ONE PASS PER CHARACTER
     '
PrintIO22:
     '
     PrintPointer=PrintPointer+1     'Advance to next character in current line
     '
     if PrintPointer>LineLength goto PrintIO23   'End of current line
     x$=mid$(PrintLine$,PrintPointer,1)          'Get next character to print
     return
     '
PrintIO23:
     '
     PrintLine=PrintLine+1                 'Advance to next line
     '                                    
     if PrintLine>LastLine         _       'If last line has been printed,
        then FinishedPrinting=1    _       'then set finished flag
            :gosub PrintIO24       _       'Delay before closing channel
            :return PrintIO8               'Exit from PrintIO.Bas
     '
     PrintLine$=PrintArray$(PrintLine)     'Get new print line
     LineLength=len(PrintLine$)            'Get length of new print line
     '
     PrintPointer=0   'Set character-within-line pointer before first character
     goto PrintIO22   'Go and get the first character of the new line
     '
     '-------------------------------------------------------------------------
     'ONE SECOND DELAY
     '
PrintIO24: PrintIOtime$=time$
PrintIO25: if PrintIOtime$=time$ goto PrintIO25
           return
     '
     '-------------------------------------------------------------------------
     '
