






     '-------------------------------------------------------------------------
     'TELEX NUMBER EXTRACTOR           03APR87               File: TelexNum.Bas 
     '-------------------------------------------------------------------------
     '
     'Author : R J Morton
     '
     'This routine scans the Telex number line in the name & address record
     'and extracts from it the prospect's Telex N or Electronic Mailbox N.
     '
     'In the prospect's Telex N field may be entered either his Telex N or
     'his electronic mailbox number, or both. The format for information in
     'the Telex Field is as follows:
     '
     '85-9147645=EBSL G {B19016965}   Spaces should be included as shown.
     '                 
     'ĳĳĳĳĳĳ International (Foreign Country) code
     '  ĳĳĳĳĳ Hyphen (only if Country Code present)
     '      ĳĳĳĳ Subscriber's National Telex N
     '            ĳĳĳ Subscriber's Answerback code
     '                ĳĳ Country identification letter
     '                   ĳ Email menu letter of subscriber's service
     '                        Subscriber's mailbox number
     '     
     'If an electronic mail N appears in the Telex Field of the subscriber's
     'name & address box, then the message is sent to him by electronic mail.
     'If not, his telex number is used to send the message to him by telex via
     'the currently selected electronic mail service. If an electronic mail
     'box number is given which does not belong to the currently selected
     'service, then the message is not sent.
     '
     'An electronic mailbox number it is prefixed by a letter A through I
     'indicating which electronic mail service it pertains to. The actual
     'services which correspond to each of the letters A through I are shown
     'on the electronic mail services selection screen.
     '
     'Input:    Telex number from Rec$(9) of the prospect name & address array
     '
     'Outputs:  TelexNum$   - Subscriber's Telex N + country code
     '          AnswerBack$ - Subscriber's (prospect's) telex answerback code
     '          EmailNum$   - Subscriber's (prospect's) electronic mailbox N
     '          ServiceId$  - Identifying letter of Prospect's Email service.
     '          DOSmsg$     - possible error message
     '
     '--------------------------------------------------------------------------
     '





     '--------------------------------------------------------------------------
     'INITIALIZATION
     '
ExtrTelexNum:
     EmailNum$=""             'Electronic Mail mailbox number
     TelexNum$=""             'Prospect's telex number + country code
     AnswerBack$=""           'Prospect's telex answer-back code
     DOSmsg$=""               'Error message variable
     DOSclr=12                'Set to display error messages in bright red
     ShowMessages=1           'Activate message display routine
     Y$=rec$(9)               'Telex N element in prospect record array
     '
     '--------------------------------------------------------------------------
     'CHOP OFF ASTERISKED COMMENT IF PRESENT [First * is the comment delimiter]
     '
     x=instr(Y$,"*")
     if x>0 then Y$=left$(Y$,x-1)
     if Y$="" then               _           'If there is nothing in the
        DOSmsg$="No Telex/FAX/Mailbox N" _  'telex number field, return with
        :return                              'the appropriate error message.
     '
     '--------------------------------------------------------------------------
     'EXTRACT THE ELECTRONIC MAILBOX NUMBER IF PRESENT
     '
     a=instr(Y$,"{")                      'Electronic mailbox left parenthesis
     b=instr(Y$,"}")                      'Electronic mailbox right parenthesis
     if a>0 or b>0 then                   'Electronic mailbox N present
        if a=0 or b=0 or b<a+3 then                 'If one brace there & other
           DOSmsg$="Syntax Error in Telex Field"    'isn't then syntax error.
      else ServiceID$=mid$(Y$,a+1,1)                'Email service I/D letter
           if ServiceID$<"A" or ServiceID$>"I" then 'If letter out of range,
              DOSmsg$="Bad Email Service I/D"       'so set up error message
       elseif ServiceID$<>chr$(ems+64) then         'If prospect not on current
              DOSmsg$="Not on "+ems$                'email service....
         else EmailNum$=mid$(Y$,a+2,b-a-2)          'Valid electronic mailbox N
              TelexNum$=EmailNum$                   'For answerback field in
              end if                                'command.
           end if
        return                                      'exit
        end if
     '
     '--------------------------------------------------------------------------
     '





     '--------------------------------------------------------------------------
     'EXTRACT THE TELEX NUMBER
     '
     L=len(Y$)                              'N of characters in Telex Field
     For K=1 to L
     X$=mid$(Y$,K,1):x=asc(X$)              'Get [next] character
     if x>96 and x<123 then x=x-32          'Smalls to caps.
     if x>47 and x<58 or x=45 then          'If a numeric or a hyphen, then
        TelexNum$=TelexNum$+X$              'add character into Telex N string
 elseif x>64 and x<91 or x=32 then          'If a letter or a space, then
        AnswerBack$=AnswerBack$+X$          'add char into AnswerCode string
        end if
     next
     if len(TelexNum$)<5 then               'If the telex number is less than
        DOSmsg$="Bad/missing Telex/FAX N"  '5 characters, it is considered bad
        return
        end if
     '
     '--------------------------------------------------------------------------
     'STRIP ALL FORE & AFT SPACES FROM THE ANSWER CODE
     '
     l=len(Answerback$):m=0:n=l
     for k=1 to l
     if mid$(Answerback$,k,1)=" " then
        if n=l then m=k
   else n=k
        end if
     next
     Answerback$=mid$(Answerback$,m+1,n-m)
     '
     if Answerback$="FAX" then EmailNum$="FAX "+TelexNum$:return
     '
     '--------------------------------------------------------------------------
     '





     '--------------------------------------------------------------------------
     'SEPARATE THE SUBSCRIBER NUMBER FROM THE COUNTRY CODE
     '
     a=instr(TelexNum$,"-")             'Country code delimiter.
     if a>0 then                        'If a foreign telen N, then
        X$="/ITX "                      'Prefix for dialling a foreign Telex N
        Y$=left$(TelexNum$,a-1)         'Extract the country code into Y$
        TelexNum$=right$(TelexNum$,len(TelexNum$)-a)        
   else X$="/TLX "                      'Prefix for dialling a UK telex number
        Y$=""                           'If code ends in " G" then it's a UK N
        end if
     '
     '--------------------------------------------------------------------------
     'FORM THE FULL DIALLING COMMAND STRING  [MERCURY LINK7500 SPECIFIC]
     '
     EmailNum$=X$            _   'National/foreign telex number command prefix
              +Y$            _   'Country code (if present)
              +TelexNum$     _   'Form the full telex address command to be
              +"("           _   'Initial bracket [Mercury Link7500]
              +TelexNum$     _   'Subscriber's Telex N excluding country code
              +AnswerBack$   _   'Answerback Code
              +")"               'Terminating bracket [Mercury]
     return                      'Return to the calling program.
     '
     '--------------------------------------------------------------------------
     '
